/* $Id: types.h,v 1.8 2000/06/20 23:48:58 becka Exp $
******************************************************************************

   KGI - general definitons, data structures, etc.

   Copyright (C) 1995-1996	Andreas Beck	[becka@ggi-project.org]
   Copyright (C) 1995-1996	Steffen Seeger	[seeger@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef	_KGI_TYPES_H
#define	_KGI_TYPES_H

#define	KB	*(1024)
#define	MB	*(1024*1024)
#define	GB	*(1024*1024*1024)

#define	BITS_PER(x)	(sizeof(x)*8)

#include <kgi/system.h>

/*\
|*|	generic data types
\*/

#define        KGI_AUTO     (-1)	/* Can be given to use defaults. */
/* the current GRAPHTYPE definition depends on KGI_AUTO == -1!!! */

typedef struct { kgis32 min, max; }				kgi_range;
typedef struct { kgis32 mul, div; }				kgi_ratio;
typedef struct { kgis16   x, y; }				kgi_coord;
typedef	struct { kgiu16   x, y; }				kgi_size;
typedef struct { kgis16	x1,y1,x2,y2; }				kgi_rect;
typedef	kgiu32							kgi_pixel;
typedef	kgiu32							kgi_attr;
	#define	ATTR_FGCOLOR	0x0000FF00	/* fgcolor clut index	*/
	#define	ATTR_BGCOLOR	0x000000FF	/* bgcolor clut index	*/
	#define	ATTR_NORMAL	0x00000000	/* normal style		*/
	#define	ATTR_HALF	0x00010000	/* half intensity	*/
	#define	ATTR_BRIGHT	0x00020000	/* high intensity	*/
	#define	ATTR_INTENSITY	0x00030000	/* mask to get intensity*/
	#define	ATTR_UNDERLINE	0x00040000	/* underline attribute	*/
	#define	ATTR_BOLD	0x00080000	/* bold style		*/
	#define	ATTR_ITALIC	0x00100000	/* italic style		*/
	#define	ATTR_REVERSE	0x00200000	/* reverse fg/bg	*/
	#define	ATTR_BLINK	0x00800000	/* enable blinking	*/
	#define	ATTR_COLOR(fg, bg)	((bg & 0xFF) | ((fg & 0xFF) << 8))

typedef	kgiu8							kgi_alpha;
	#define	KGI_MAX_ALPHA	255
typedef	kgiu16							kgi_zvalue;
typedef struct { kgiu16 r,g,b; }				kgi_color;
typedef struct { kgiu16	size; kgi_color	*data; }		kgi_clut;
#define	KGI_COLOR_PRECISION 16	/* 16 bit per R,G, B value	*/

/* FIXME ! Don't use shorts here ! Needs conversion ! See C argument expansion rules ! */
typedef struct { short x,y; kgiu16 a,b; short a1,a2; }	kgi_arc_data;
typedef struct { short x,y; }					kgi_point_data;
typedef struct { short x1,y1; short x2,y2; }			kgi_line_data;
typedef struct { short x,y; kgiu16 w,h; }		kgi_rect_data;

typedef enum
{	KGIGT_TEXT16,		/* 16 bpp alphanumeric modes	*/
	KGIGT_TEXT32,		/* 32 bpp alphanumeric modes	*/
	KGIGT_1BIT,		/*  1 bpp graphics		*/
	KGIGT_4BIT,		/*  4 bpp graphics		*/
	KGIGT_8BIT,		/*  8 bpp graphics		*/
	KGIGT_15BIT,		/* 15 bpp graphics 		*/
	KGIGT_16BIT,		/* 16 bpp graphics		*/
	KGIGT_24BIT,		/* 24 bpp graphics		*/
	KGIGT_32BIT,		/* 24 bpp word aligned  	*/
	KGIGT_LAST			/* last mode assigned   	*/
} kgi_graphtype;

#define KGI_IS_TEXTMODE(mode) ((mode)->graphtype<=KGIGT_TEXT32)

#define MASK(X) GM_##X = (1 << KGIGT_##X) 	
typedef enum
{	MASK(TEXT16),
	MASK(TEXT32),
	MASK(1BIT),
	MASK(4BIT),
	MASK(8BIT),
	MASK(15BIT),
	MASK(16BIT),
	MASK(24BIT),
	MASK(32BIT),
	GM_ALL	=  ((1 << KGIGT_LAST) - 1)
} kgi_graphmask;
#undef MASK

typedef enum {
	KGICM_AUTO,	/* autoselect */
	KGICM_RGB,	/* RGB  - red geern blue additive model */
	KGICM_YUV,	/* YUV  - for video */
	KGICM_CMYK,	/* CMYK - subtractive model */
	KGICM_LAST
} kgi_colormodel;

typedef struct		/* requested by user and changed by driver    */
{
	kgiu32		frames;		/* frames needed			*/
	kgi_coord	visible;	/* vis. pixels, may change slightly	*/
	kgi_coord	virt;		/* virtual pixels, may change		*/
	kgi_coord	size;		/* size of visible in mm		*/
	kgi_graphtype	graphtype;	/* which mode ?				*/
	kgi_coord	dpp;		/* dots per pixel			*/
} kgi_modereq;

/*\
|*|	drawables and displays
\*/

#include <kgi/font.h>

typedef enum
{	MAP_NOTHING,			/* ignore all op's (must be 0)	*/
	MAP_HARDWARE,			/* use hardware accel only	*/
	MAP_HARD_MEM,			/* mmap'ed hardware or accel	*/
	MAP_MEMORY,			/* use only a memory buffer	*/
	MAP_DRIVER			/* use ioctl interface		*/
} kgi_mapping;

typedef enum
{	MAP_256_BPP,
	MAP_128_BPP,
	MAP_64_BPP,
	MAP_32_BPP,
	MAP_16_BPP,
	MAP_8_BPP,
	MAP_4_BPP,
	MAP_2_BPP,
	MAP_1_BPP
} kgi_maptype;

/*\
|*|	drawing context
\*/

typedef struct kgi_gc
{
	kgi_uint	version;
	kgi_pixel	fg_color;	/* foreground pixel value	*/
	kgi_pixel	bg_color;	/* background pixel value	*/
	kgi_coord	cliptl,	clipbr;	/* clipping rectangle		*/
        kgi_pixel	plane_mask;	/* plane mask bits		*/
	/* KGI internal fields */
	kgi_uint	oldversion;	/* Version last time we checked */
} kgi_gc;

#endif	/* #ifdef _KGI_TYPES_H */
